@@ -24,7 +24,7 @@ add_data <- function(p, data = NULL) {
 #' @inheritParams plot_ly
 #' @param p a plotly object
 #' @param inherit inherit attributes from [plot_ly()]?
-#' @param z a numeric matrix
+#' @param z a numeric matrix (unless [add_image()], which wants a raster object, see [as.raster()]).
 #' @param x the x variable.
 #' @param y the y variable.
 #' @param text textual labels.
@@ -33,11 +33,11 @@ add_data <- function(p, data = NULL) {
 #' @param xend "final" x position (in this context, x represents "start")
 #' @param yend "final" y position (in this context, y represents "start")
 #' @seealso [plot_ly()]
-#' @references \url{http://plotly-book.cpsievert.me/the-plotly-cookbook.html}
+#' @references \url{https://plotly-r.com/overview.html}
 #' 
-#' \url{https://plot.ly/r}
+#' \url{https://plotly.com/r/}
 #' 
-#' \url{https://plot.ly/r/reference/} 
+#' \url{https://plotly.com/r/reference/} 
 #' @author Carson Sievert
 #' @export
 #' @rdname add_trace
@@ -130,9 +130,14 @@ add_data <- function(p, data = NULL) {
 #'   layout(title = "Basic Pie Chart using Plotly")
 #'   
 #' data(wind)
-#' plot_ly(wind, r = ~r, t = ~t) %>% 
+#' plot_ly(wind, r = ~r, theta = ~t) %>% 
 #'   add_area(color = ~nms) %>%
-#'   layout(radialaxis = list(ticksuffix = "%"), orientation = 270)
+#'   layout(
+#'     polar = list(
+#'       radialaxis = list(ticksuffix = "%"), 
+#'       angularaxis = list(rotation = 90)
+#'     )
+#'   )
 #' 
 #' # ------------------------------------------------------------
 #' # 3D chart types
@@ -334,10 +339,11 @@ add_sf <- function(p, ..., x = ~x, y = ~y, data = NULL, inherit = TRUE) {
       y = y,
       `_bbox` = bbox,
       set = set,
-      data = if ("group" %in% names(d[[i]])) group_by_(d[[i]], "group", add = TRUE) else d[[i]], 
+      data = if ("group" %in% names(d[[i]])) group_by_add(d[[i]], !!rlang::sym("group")) else d[[i]], 
       inherit = inherit
     )
-    p <- do.call(add_trace_classed, c(args, attrs))
+    args <- modify_list(args, attrs)
+    p <- do.call(add_trace_classed, args)
   }
   
   p
@@ -394,20 +400,66 @@ add_ribbons <- function(p, x = NULL, ymin = NULL, ymax = NULL, ...,
 
 #' @inheritParams add_trace
 #' @rdname add_trace
-#' @param r For polar chart only. Sets the radial coordinates.
-#' @param t For polar chart only. Sets the radial coordinates.
+#' @param colormodel Sets the colormodel for image traces if `z` is not a raster object. 
+#' If `z` is a raster object (see [as.raster()]), the `'rgba'` colormodel is always used.
 #' @export
-add_area <- function(p, r = NULL, t = NULL, ...,
-                     data = NULL, inherit = TRUE) {
+add_image <- function(p, z = NULL, colormodel = NULL, ..., data = NULL, inherit = TRUE) {
+  
   if (inherit) {
-    r <- t %||% p$x$attrs[[1]][["r"]]
-    t <- t %||% p$x$attrs[[1]][["t"]]
+    z <- z %||% p$x$attrs[[1]][["z"]]
+    colormodel <- colormodel %||% p$x$attrs[[1]][["colormodel"]]
   }
-  if (is.null(r) || is.null(t)) {
-    stop("Must supply `r`/`t` attributes", call. = FALSE)
+  
+  if (inherits(z, "raster")) {
+    cols <- col2rgb(z, alpha = TRUE)
+    dims <- c(dim(z), 4)
+    z <- array(numeric(prod(dims)), dims)
+    matrix_ <- function(x) {
+      matrix(x, byrow = TRUE, nrow = dims[1], ncol = dims[2])
+    }
+    z[,,1] <- matrix_(cols["red",])
+    z[,,2] <- matrix_(cols["green",])
+    z[,,3] <- matrix_(cols["blue",])
+    z[,,4] <- matrix_(cols["alpha",])
+    
+    # Throw if we detect another colormodel
+    if (!identical(colormodel %||% "rgba", "rgba")) {
+      warning("Passing a raster object to z requires rgba colormodel")
+    }
+    colormodel <- "rgba"
+  }
+  
+  add_trace(
+    p, z = z, colormodel = colormodel, ..., 
+    data = data, type = "image"
+  )
+}
+
+#' @inheritParams add_trace
+#' @rdname add_trace
+#' @param r Sets the radial coordinates.
+#' @param theta Sets the angular coordinates.
+#' @param t Deprecated. Use `theta` instead.
+#' @export
+add_area <- function(p, r = NULL, theta = NULL, t = NULL, ...,
+                     data = NULL, inherit = TRUE) {
+  if (!is.null(t)) {
+    message(
+      "Since `add_area()` now uses the barpolar instead of the area trace, ", 
+      "the `t` argument is now deprecated. Use the `theta` argument instead."
+    )
+  }
+  theta <- theta %||% t
+  if (inherit) {
+    attrs <- p$x$attrs[[1]]
+    r <- r %||% attrs[["r"]]
+    theta <- theta %||% (attrs[["theta"]] %||% attrs[["t"]])
+  }
+  if (is.null(r) || is.null(theta)) {
+    stop("Must supply `r`/`theta` attributes", call. = FALSE)
   }
   add_trace_classed(
-    p, class = "plotly_area", r = r, t = t, type = "area",
+    p, class = "plotly_area", r = r, theta = theta, type = "barpolar",
     ..., data = data, inherit = inherit
   )
 }
