@@ -264,7 +264,12 @@ gg2list <- function(p, width = NULL, height = NULL,
     layers <- Map(function(x, y) {
       if (crosstalk_key() %in% names(y) && !"key" %in% names(x[["mapping"]]) && 
           inherits(x[["stat"]], "StatIdentity")) {
-        x[["mapping"]] <- c(x[["mapping"]], key = as.name(crosstalk_key()))
+        # ggplot2 v3.3.4 started using the computed_mapping (instead of mapping)
+        # field to inform the compute_aesthetics() method, so in order to add
+        # the crosstalk key, we need to add to that field (when present)
+        # https://github.com/tidyverse/ggplot2/pull/4475
+        nm <- if ("computed_mapping" %in% names(x)) "computed_mapping" else "mapping"
+        x[[nm]] <- c(x[[nm]], key = as.name(crosstalk_key()))
       }
       x
     }, layers, layer_data)
@@ -341,7 +346,7 @@ gg2list <- function(p, width = NULL, height = NULL,
       x <- reComputeGroup(x, z)
       tib <- tibble::as_tibble(x[c("PANEL", "group")])
       tib[["key"]] <- key
-      nested <- tidyr::nest(tib, key, .key = key)
+      nested <- tidyr::nest(tib, key = key)
       # reduce the dimensions of list column elements from 2 to 1
       nested$key <- lapply(nested$key, function(x) x[[1]])
       nested
@@ -828,11 +833,12 @@ gg2list <- function(p, width = NULL, height = NULL,
       gglayout[[axisName]] <- axisObj
       
       # do some stuff that should be done once for the entire plot
+      is_x <- xy == "x"
       if (i == 1) {
         # Split ticktext elements by "\n"  to account for linebreaks
         axisTickText <- strsplit(as.character(axisObj$ticktext), split = "\n", fixed = TRUE)
         axisTickText <- longest_element(unlist(axisTickText))
-        side <- if (xy == "x") "b" else "l"
+        side <- if (is_x) "b" else "l"
         # account for axis ticks, ticks text, and titles in plot margins
         # (apparently ggplot2 doesn't support axis.title/axis.text margins)
         gglayout$margin[[side]] <- gglayout$margin[[side]] + axisObj$ticklen +
@@ -842,40 +848,45 @@ gg2list <- function(p, width = NULL, height = NULL,
         if (robust_nchar(axisTitleText) > 0) {
           axisTextSize <- unitConvert(axisText, "npc", type)
           axisTitleSize <- unitConvert(axisTitle, "npc", type)
-          offset <-
-            (0 -
-               bbox(axisTickText, axisText$angle, axisTextSize)[[type]] -
-               bbox(axisTitleText, axisTitle$angle, axisTitleSize)[[type]] / 2 -
-               unitConvert(theme$axis.ticks.length, "npc", type))
         }
         
         # add space for exterior facet strips in `layout.margin`
         
         if (has_facet(plot)) {
           stripSize <- unitConvert(stripText, "pixels", type)
-          if (xy == "x") {
+          if (is_x) {
             gglayout$margin$t <- gglayout$margin$t + stripSize
           }
-          if (xy == "y" && inherits(plot$facet, "FacetGrid")) {
+          if (is_x && inherits(plot$facet, "FacetGrid")) {
             gglayout$margin$r <- gglayout$margin$r + stripSize
           }
           # facets have multiple axis objects, but only one title for the plot,
           # so we empty the titles and try to draw the title as an annotation
           if (robust_nchar(axisTitleText) > 0) {
-            # npc is on a 0-1 scale of the _entire_ device,
-            # but these units _should_ be wrt to the plotting region
-            # multiplying the offset by 2 seems to work, but this is a terrible hack
-            x <- if (xy == "x") 0.5 else offset
-            y <- if (xy == "x") offset else 0.5
-            gglayout$annotations <- c(
-              gglayout$annotations,
-              make_label(
-                faced(axisTitleText, axisTitle$face), x, y, el = axisTitle,
-                xanchor = if (xy == "x") "center" else "right", 
-                yanchor = if (xy == "x") "top" else "center", 
-                annotationType = "axis"
-              )
+            axisAnn <- make_label(
+              faced(axisTitleText, axisTitle$face), 
+              el = axisTitle,
+              x = if (is_x) 0.5 else 0,
+              y = if (is_x) 0 else 0.5,
+              xanchor = if (is_x) "center" else "right", 
+              yanchor = if (is_x) "top" else "center", 
+              annotationType = "axis"
             )
+            
+            textMargin <- sum(axisText$margin[if (is_x) c(1, 3) else c(2, 4)])
+            class(textMargin) <- setdiff(class(textMargin), "margin")
+            titleMargin <- axisTitle$margin[if (is_x) 1 else 2]
+            class(titleMargin) <- setdiff(class(titleMargin), "margin")
+            offset <- bbox(axisTickText, axisText$angle, axisTextSize)[[type]] +
+                 unitConvert(theme$axis.ticks.length, "npc", type) +
+                 unitConvert(textMargin, "npc", type) +
+                 unitConvert(titleMargin, "npc", type)
+            
+            offset <- unitConvert(grid::unit(offset, "npc"), "pixels", type)
+            
+            shift <- if (is_x) "yshift" else "xshift"
+            axisAnn[[1]][[shift]] <- -1 * offset
+            gglayout$annotations <- c(gglayout$annotations, axisAnn)
           }
         }
       }
@@ -994,26 +1005,12 @@ gg2list <- function(p, width = NULL, height = NULL,
     }
     traces <- c(traces, colorbar)
     
-    # legend title annotation - https://github.com/plotly/plotly.js/issues/276
-    if (isTRUE(gglayout$showlegend)) {
-      legendTitles <- compact(lapply(gdefs, function(g) if (inherits(g, "legend")) g$title else NULL))
-      legendTitle <- paste(legendTitles, collapse = br())
-      titleAnnotation <- make_label(
-        legendTitle,
-        x = gglayout$legend$x %||% 1.02,
-        y = gglayout$legend$y %||% 1,
-        theme$legend.title,
-        xanchor = "left",
-        yanchor = "bottom",
-        # just so the R client knows this is a title
-        legendTitle = TRUE
-      )
-      gglayout$annotations <- c(gglayout$annotations, titleAnnotation)
-      # adjust the height of the legend to accomodate for the title
-      # this assumes the legend always appears below colorbars
-      gglayout$legend$y <- (gglayout$legend$y %||% 1) -
-        length(legendTitles) * unitConvert(theme$legend.title$size, "npc", "height")
-    }
+    legendTitles <- compact(lapply(gdefs, function(g) if (inherits(g, "legend")) g$title else NULL))
+    legendTitle <- paste(legendTitles, collapse = br())
+    gglayout$legend$title <- list(
+      text = legendTitle,
+      font = text2font(theme$legend.title)
+    )
   }
   
   # flip x/y in traces for flipped coordinates
@@ -1099,7 +1096,7 @@ gg2list <- function(p, width = NULL, height = NULL,
   # start build a plotly object with meta information about the ggplot
   # first, translate layer mappings -> plotly attrs
   mappingFormulas <- lapply(layers, function(x) {
-    mappings <- c(x$mapping, if (isTRUE(x$inherit.aes)) plot$mapping)
+    mappings <- getAesMap(plot, x)
     if (originalData) {
       lapply(mappings, lazyeval::f_new)
     } else {
@@ -1427,3 +1424,12 @@ gdef2trace <- function(gdef, theme, gglayout) {
     NULL
   }
 }
+
+
+getAesMap <- function(plot, layer) {
+  if (isTRUE(layer$inherit.aes)) {
+    modify_list(plot$mapping, layer$mapping)
+  } else {
+    layer$mapping
+  }
+}
