@@ -6,16 +6,19 @@ layers2traces <- function(data, prestats_data, layout, p) {
   # Extract parameters (and "hovertext aesthetics") in each layer
   params <- Map(function(x, y) {
     param <- c(
-      y[["geom_params"]], y[["stat_params"]], y[["aes_params"]], 
+      y[["computed_geom_params"]] %||% y[["geom_params"]], 
+      y[["computed_stat_params"]] %||% y[["stat_params"]], 
+      y[["aes_params"]], 
       position = ggtype(y, "position")
     )
     
+    map <- getAesMap(p, y)
+    
     # consider "calculated" aesthetics (e.g., density, count, etc)
     calc_aes <- y$stat$default_aes[ggfun("is_calculated_aes")(y$stat$default_aes)]
-    map <- c(y$mapping, calc_aes)
+    calc_aes <- calc_aes[!names(calc_aes) %in% names(map)]
     
-    # add on plot-level mappings, if they're inherited
-    if (isTRUE(y$inherit.aes)) map <- c(map, p$mapping)
+    map <- c(calc_aes, map)
     
     # turn symbol (e.g., ..count..) & call (e.g. calc(count)) mappings into text labels 
     map <- ggfun("make_labels")(map)
@@ -271,16 +274,14 @@ to_basic.GeomRect <- function(data, prestats_data, layout, params, p, ...) {
 
 #' @export
 to_basic.GeomSf <- function(data, prestats_data, layout, params, p, ...) {
-  
-  data[["geometry"]] <- sf::st_sfc(data[["geometry"]])
-  data <- sf::st_as_sf(data, sf_column_name = "geometry")
+
+  data <- sf::st_as_sf(data)
   geom_type <- sf::st_geometry_type(data)
   # st_cast should "expand" a collection into multiple rows (one per feature)
   if ("GEOMETRYCOLLECTION" %in% geom_type) {
     data <- sf::st_cast(data)
     geom_type <- sf::st_geometry_type(data)
   }
-  data <- remove_class(data, "sf")
   
   basic_type <- dplyr::recode(
     as.character(geom_type),
@@ -308,6 +309,7 @@ to_basic.GeomSf <- function(data, prestats_data, layout, params, p, ...) {
     d[[i]] <- prefix_class(
       fortify_sf(d[[i]]), c(names(d)[[i]], "GeomSf")
     )
+    d[[i]] <- remove_class(d[[i]], "sf")
   }
   if (length(d) == 1) d[[1]] else d
 }
@@ -417,8 +419,10 @@ to_basic.GeomHline <- function(data, prestats_data, layout, params, p, ...) {
   data$group <- do.call(paste,
     data[!grepl("group", names(data)) & !vapply(data, anyNA, logical(1))]
   )
-  lay <- tidyr::gather_(layout$layout, "variable", "x", c("x_min", "x_max"))
-  data <- merge(lay[c("PANEL", "x")], data, by = "PANEL")
+  x <- if (inherits(p$coordinates, "CoordFlip")) "y" else "x"
+  lay <- tidyr::gather_(layout$layout, "variable", x, paste0(x, c("_min", "_max")))
+  data <- merge(lay[c("PANEL", x)], data, by = "PANEL")
+  data[["x"]] <- data[[x]]
   data[["y"]] <- data$yintercept
   prefix_class(data, c("GeomHline", "GeomPath"))
 }
@@ -429,8 +433,10 @@ to_basic.GeomVline <- function(data, prestats_data, layout, params, p, ...) {
   data$group <- do.call(paste,
     data[!grepl("group", names(data)) & !vapply(data, anyNA, logical(1))]
   )
-  lay <- tidyr::gather_(layout$layout, "variable", "y", c("y_min", "y_max"))
-  data <- merge(lay[c("PANEL", "y")], data, by = "PANEL")
+  y <- if (inherits(p$coordinates, "CoordFlip")) "x" else "y"
+  lay <- tidyr::gather_(layout$layout, "variable", y, paste0(y, c("_min", "_max")))
+  data <- merge(lay[c("PANEL", y)], data, by = "PANEL")
+  data[["y"]] <- data[[y]]
   data[["x"]] <- data$xintercept
   prefix_class(data, c("GeomVline", "GeomPath"))
 }
@@ -446,7 +452,11 @@ to_basic.GeomErrorbar <- function(data, prestats_data, layout, params, p, ...) {
   # width for ggplot2 means size of the entire bar, on the data scale
   # (plotly.js wants half, in pixels)
   data <- merge(data, layout$layout, by = "PANEL", sort = FALSE)
-  data$width <- (data[["xmax"]] - data[["x"]]) /(data[["x_max"]] - data[["x_min"]])
+  data$width <- if (params[["flipped_aes"]]) {
+    (data[["ymax"]] - data[["y"]]) /(data[["y_max"]] - data[["y_min"]])  
+  } else {
+    (data[["xmax"]] - data[["x"]]) /(data[["x_max"]] - data[["x_min"]])
+  }
   data$fill <- NULL
   prefix_class(data, "GeomErrorbar")
 }
@@ -528,7 +538,7 @@ to_basic.GeomCrossbar <- function(data, prestats_data, layout, params, p, ...) {
     prefix_class(to_basic.GeomSegment(middle), "GeomCrossbar")
   )
 }
-utils::globalVariables(c("xmin", "xmax", "y", "size"))
+utils::globalVariables(c("xmin", "xmax", "y", "size", "COL", "PANEL", "ROW", "yaxis"))
 
 #' @export
 to_basic.GeomRug  <- function(data, prestats_data, layout, params, p, ...) {
@@ -637,6 +647,7 @@ geom2trace.GeomPath <- function(data, params, p) {
     y = data[["y"]],
     text = uniq(data[["hovertext"]]),
     key = data[["key"]],
+    customdata = data[["customdata"]],
     frame = data[["frame"]],
     ids = data[["ids"]],
     type = "scatter",
@@ -667,6 +678,7 @@ geom2trace.GeomPoint <- function(data, params, p) {
     y = data[["y"]],
     text = if (isDotPlot) data[["key"]] else uniq(data[["hovertext"]]),
     key = data[["key"]],
+    customdata = data[["customdata"]],
     frame = data[["frame"]],
     ids = data[["ids"]],
     type = "scatter",
@@ -720,9 +732,15 @@ geom2trace.GeomBar <- function(data, params, p) {
     y = y,
     text = uniq(data[["hovertext"]]),
     key = data[["key"]],
+    customdata = data[["customdata"]],
     frame = data[["frame"]],
     ids = data[["ids"]],
     type = "bar",
+    # plotly.js v2.0 changed default to textposition='auto', meaning
+    # text will display by default, which makes sense for plot_ly() maybe, 
+    # but not ggplotly()
+    # https://github.com/plotly/orca/issues/374
+    textposition = "none",
     marker = list(
       autocolorscale = FALSE,
       color = toRGB(
@@ -747,6 +765,7 @@ geom2trace.GeomPolygon <- function(data, params, p) {
     y = data[["y"]],
     text = uniq(data[["hovertext"]]),
     key = data[["key"]],
+    customdata = data[["customdata"]],
     frame = data[["frame"]],
     ids = data[["ids"]],
     type = "scatter",
@@ -778,6 +797,7 @@ geom2trace.GeomBoxplot <- function(data, params, p) {
     y = data[["y"]],
     hoverinfo = "y",
     key = data[["key"]],
+    customdata = data[["customdata"]],
     frame = data[["frame"]],
     ids = data[["ids"]],
     type = "box",
@@ -812,6 +832,7 @@ geom2trace.GeomText <- function(data, params, p) {
     text = data[["label"]],
     hovertext = data[["hovertext"]],
     key = data[["key"]],
+    customdata = data[["customdata"]],
     frame = data[["frame"]],
     ids = data[["ids"]],
     textfont = list(
@@ -850,6 +871,7 @@ geom2trace.GeomTile <- function(data, params, p) {
     z = matrix(g$fill_plotlyDomain, nrow = length(y), ncol = length(x), byrow = TRUE),
     text = matrix(g$hovertext, nrow = length(y), ncol = length(x), byrow = TRUE),
     key = data[["key"]],
+    customdata = data[["customdata"]],
     frame = data[["frame"]],
     ids = data[["ids"]],
     colorscale = setNames(colScale, NULL),
@@ -861,7 +883,17 @@ geom2trace.GeomTile <- function(data, params, p) {
 
 #' @export
 geom2trace.GeomErrorbar <- function(data, params, p) {
-  make_error(data, params, "y")
+  # Support of bi-directional GeomErrorbar introduced with ggplot2 3.3.0
+  # g <- ggplot() + geom_errorbar(aes(y = "A", xmin = 1, xmax = 2))
+  # ggplotly(g)
+# Support of bi-directional GeomErrorbar introduced with ggplot2 3.3.0:
+# g <- ggplot() + geom_errorbar(aes(y = "A", xmin = 1, xmax = 2))
+# ggplotly(g)
+if (params[["flipped_aes"]]) { 
+    make_error(data, params, "x")
+  } else {
+    make_error(data, params, "y")
+  }
 }
 
 #' @export
@@ -939,12 +971,15 @@ hover_on <- function(data) {
 
 # make trace with errorbars
 make_error <- function(data, params, xy = "x") {
+  # if xy is NULL: set xy to mean of xy_min and xy_max
+  data[[xy]] <- data[[xy]] %||% ((data[[paste0(xy, "min")]] + data[[paste0(xy, "max")]]) / 2)  
   color <- aes2plotly(data, params, "colour")
   e <- list(
     x = data[["x"]],
     y = data[["y"]],
     text = uniq(data[["hovertext"]]),
     key = data[["key"]],
+    customdata = data[["customdata"]],
     frame = data[["frame"]],
     ids = data[["ids"]],
     type = "scatter",
@@ -992,7 +1027,12 @@ aes2plotly <- function(data, params, aes = "size") {
     type <- if (any(grepl("point", class(data)))) "point" else if (any(grepl("line", class(data)))) "line" else ""
     ggfun("default_aesthetics")(type)
   } else {
-    ggfun(geom)$default_aes
+    geom_obj <- ggfun(geom)
+    # If the first class of `data` is a data.frame,
+    # ggfun() returns a function because ggplot2 now
+    # defines data.frame in it's namespace
+    # https://github.com/ropensci/plotly/pull/1481
+    if ("default_aes" %in% names(geom_obj)) geom_obj$default_aes else NULL
   }
   
   vals <- uniq(data[[aes]]) %||% params[[aes]] %||% defaults[[aes]] %||% NA
