@@ -57,13 +57,13 @@
 #'   do(p = plot_ly(., x = ~date, y = ~value)) %>%
 #'   subplot(nrows = NROW(.), shareX = TRUE)
 #'   
-#' # learn more at https://cpsievert.github.io/plotly_book/subplot.html
+#' # learn more at https://plotly.com/r/subplot.html
 #' 
 
 subplot <- function(..., nrows = 1, widths = NULL, heights = NULL, margin = 0.02, 
                     shareX = FALSE, shareY = FALSE, titleX = shareX, 
                     titleY = shareY, which_layout = "merge") {
- 
+  
   
   plots <- dots2plots(...)
   
@@ -86,7 +86,7 @@ subplot <- function(..., nrows = 1, widths = NULL, heights = NULL, margin = 0.02
   # Are any traces referencing "axislike" layout attributes that are missing?
   # If so, move those traces to a "new plot", and inherit layout attributes,
   # which makes this sort of thing possible:
-  # https://plot.ly/r/map-subplots-and-small-multiples/
+  # https://plotly.com/r/map-subplots-and-small-multiples/
   plots <- list()
   for (i in seq_along(plotz)) {
     p <- plots[[i]] <- plotz[[i]]
@@ -120,6 +120,7 @@ subplot <- function(..., nrows = 1, widths = NULL, heights = NULL, margin = 0.02
   traces <- lapply(plots, "[[", "data")
   layouts <- lapply(plots, "[[", "layout")
   shapes <- lapply(layouts, "[[", "shapes")
+  images <- lapply(layouts, "[[", "images")
   annotations <- lapply(layouts, function(x) {
     # keep non axis title annotations (for rescaling)
     axes <- vapply(x$annotations, function(a) identical(a$annotationType, "axis"), logical(1))
@@ -206,15 +207,34 @@ subplot <- function(..., nrows = 1, widths = NULL, heights = NULL, margin = 0.02
       map <- xMap[xMap %in% sub("x", "xaxis", yAxes[[i]][[j]]$anchor %||% "x")]
       yAxes[[i]][[j]]$anchor <- sub("axis", "", names(map))
     }
-    # map trace xaxis/yaxis/geo attributes
+    
     for (key in c("geo", "subplot", "xaxis", "yaxis")) {
+      # bump trace axis references
       oldAnchors <- unlist(lapply(traces[[i]], "[[", key))
       if (!length(oldAnchors)) next
       axisMap <- if (key == "yaxis") yMap else xMap
       axisMap <- setNames(sub("axis", "", axisMap), sub("axis", "", names(axisMap)))
       newAnchors <- names(axisMap)[match(oldAnchors, axisMap)]
       traces[[i]] <- Map(function(tr, a) { tr[[key]] <- a; tr }, traces[[i]], newAnchors)
+      
+      # bump annotation, image, shape xref/yref
+      # (none of these layout components have geo/subplot support)
+      ref <- list(xaxis = "xref", yaxis = "yref")[[key]]
+      if (is.null(ref)) next
+      bump_axis_ref <- function(obj, ref_default = sub("ref", "", ref)) {
+        # TODO: throw error/warning if ref_default doesn't match axisMap?
+        obj[[ref]] <- obj[[ref]] %||% ref_default
+        if (identical(obj[[ref]], "paper")) return(obj)
+        refIdx <- match(obj[[ref]], axisMap)
+        if (!is.na(refIdx)) obj[[ref]] <- names(axisMap)[refIdx][1]
+        obj
+      }
+      annotations[[i]] <- lapply(annotations[[i]], bump_axis_ref)
+      shapes[[i]] <- lapply(shapes[[i]], bump_axis_ref)
+      images[[i]] <- lapply(images[[i]], bump_axis_ref, "paper")
     }
+    
+    
     # rescale domains according to the tabular layout
     xDom <- as.numeric(domainInfo[i, c("xstart", "xend")])
     yDom <- as.numeric(domainInfo[i, c("yend", "ystart")])
@@ -246,7 +266,7 @@ subplot <- function(..., nrows = 1, widths = NULL, heights = NULL, margin = 0.02
   }
   
   p <- list(
-    data = Reduce(c, traces),
+    data = unlist(traces, recursive = FALSE),
     layout = Reduce(modify_list, c(xAxes, rev(yAxes)))
   )
   
@@ -254,11 +274,14 @@ subplot <- function(..., nrows = 1, widths = NULL, heights = NULL, margin = 0.02
   # reposition shapes and annotations
   annotations <- Map(reposition, annotations, split(domainInfo, seq_along(plots)))
   shapes <- Map(reposition, shapes, split(domainInfo, seq_along(plots)))
-  p$layout$annotations <- Reduce(c, annotations)
-  p$layout$shapes <- Reduce(c, shapes)
+  images <- Map(reposition, images, split(domainInfo, seq_along(plots)))
+  p$layout$annotations <- unlist(annotations, recursive = FALSE)
+  p$layout$shapes <- unlist(shapes, recursive = FALSE)
+  p$layout$images <- unlist(images, recursive = FALSE)
+
   # merge non-axis layout stuff
   layouts <- lapply(layouts, function(x) {
-    x[!grepl("^[x-y]axis|^geo|^mapbox|annotations|shapes", names(x))] %||% list()
+    x[!grepl("^[x-y]axis|^geo|^mapbox|annotations|shapes|images", names(x))] %||% list()
   })
   if (which_layout != "merge") {
     if (!is.numeric(which_layout)) warning("which_layout must be numeric")
@@ -268,8 +291,8 @@ subplot <- function(..., nrows = 1, widths = NULL, heights = NULL, margin = 0.02
     }
     layouts <- layouts[which_layout]
   }
-  p$attrs <- Reduce(c, lapply(plots, "[[", "attrs"))
-  p$layout <- modify_list(p$layout, Reduce(modify_list, layouts))
+  p$attrs <- unlist(lapply(plots, "[[", "attrs"), recursive = FALSE)
+  p$layout <- Reduce(modify_list, layouts, p$layout)
   p$source <- ensure_one(plots, "source")
   p$config <- ensure_one(plots, "config")
   p$highlight <- ensure_one(plots, "highlight")
@@ -316,14 +339,18 @@ dots2plots <- function(...) {
 # helper function that warns if more than one plot-level attribute 
 # has been specified in a list of plots (and returning that attribute)
 ensure_one <- function(plots, attr) {
-  attrs <- lapply(plots, "[", attr)
+  attrs <- Filter(Negate(is.null), lapply(plots, "[[", attr))
+  if (length(attrs) == 0) {
+    warning("No ", attr, " found", call. = FALSE)
+    return (NULL)
+  }
   for (i in seq_along(attrs)) {
     if (!identical(attrs[[1]], attrs[[i]])) {
       warning("Can only have one: ", attr, call. = FALSE)
       break
     }
   }
-  attrs[[length(attrs)]][[1]]
+  attrs[[length(attrs)]]
 }
 
 
@@ -345,7 +372,7 @@ get_domains <- function(nplots = 1, nrows = 1, margins = 0.01,
   if (any(widths < 0) | any(heights < 0)) {
     stop("The widths and heights arguments must contain positive values")
   }
-  if (sum(widths) > 1 | sum(heights) > 1) {
+  if (sum(widths) > (1 + sqrt(.Machine$double.eps)) | sum(heights) > (1 + sqrt(.Machine$double.eps))) {
     stop("The sum of the widths and heights arguments must be less than 1")
   }
   
@@ -377,7 +404,7 @@ get_domains <- function(nplots = 1, nrows = 1, margins = 0.01,
 
 list2df <- function(x, nms) {
   #stopifnot(length(unique(sapply(x, length))) == 1)
-  m <- if (length(x) == 1) t(x[[1]]) else Reduce(rbind, x)
+  m <- if (length(x) == 1) t(x[[1]]) else do.call(rbind, x)
   row.names(m) <- NULL
   df <- data.frame(m)
   if (!missing(nms)) setNames(df, nms) else df
@@ -387,22 +414,27 @@ list2df <- function(x, nms) {
 # (useful mostly for repositioning annotations/shapes in subplots)
 reposition <- function(obj, domains) {
   # we need x and y in order to rescale them!
+  xdom <- as.numeric(domains[c("xstart", "xend")])
+  ydom <- as.numeric(domains[c("yend", "ystart")])
+  
   for (i in seq_along(obj)) {
     o <- obj[[i]]
-    # TODO: this implementation currently assumes xref/yref == "paper"
-    # should we support references to axis objects as well?
-    for (j in c("x", "x0", "x1")) {
+    xs <- if (identical(o$xref, "paper")) {
+      if (is.numeric(o$sizex)) obj[[i]]$sizex <- o$sizex * abs(diff(xdom))
+      if (identical(o$xsizemode, "pixel")) "xanchor" else c("x", "x0", "x1")
+    }
+    for (j in xs) {
       if (is.numeric(o[[j]])) {
-        obj[[i]][[j]] <- scales::rescale(
-          o[[j]], as.numeric(domains[c("xstart", "xend")]), from = c(0, 1)
-        )
+        obj[[i]][[j]] <- scales::rescale(o[[j]], xdom, from = c(0, 1))
       }
     }
-    for (j in c("y", "y0", "y1")) {
+    ys <- if (identical(o$yref, "paper")) {
+      if (is.numeric(o$sizey)) obj[[i]]$sizey <- o$sizey * abs(diff(ydom))
+      if (identical(o$ysizemode, "pixel")) "yanchor" else c("y", "y0", "y1")
+    }
+    for (j in ys) {
       if (is.numeric(o[[j]])) {
-        obj[[i]][[j]] <- scales::rescale(
-          o[[j]], as.numeric(domains[c("yend", "ystart")]), from = c(0, 1)
-        )
+        obj[[i]][[j]] <- scales::rescale(o[[j]], ydom, from = c(0, 1))
       }
     }
   }
