@@ -1,7 +1,7 @@
-context("geom_sf")
 
 test_that("geom_sf() basic polygons.", {
   skip_if_not_installed("sf")
+  skip_if_not_installed("s2")
   
   nc <- sf::st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
   
@@ -37,6 +37,7 @@ test_that("geom_sf() geometry collection.", {
 
 test_that("geom_sf() polygons with fill/text.", {
   skip_if_not_installed("sf")
+  skip_if_not_installed("s2")
   
   nc <- sf::st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
   p <- ggplot(nc) + geom_sf(aes(fill = AREA, text = NAME))
@@ -56,6 +57,7 @@ test_that("geom_sf() polygons with fill/text.", {
 
 test_that("geom_sf() with basic polygons and points.", {
   skip_if_not_installed("sf")
+  skip_if_not_installed("s2")
   
   nc <- sf::st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
   p <- ggplot(nc) +
@@ -74,6 +76,7 @@ test_that("geom_sf() with basic polygons and points.", {
 
 test_that("sf aspect ratio is correct", {
   skip_if_not_installed("sf")
+  skip_if_not_installed("s2")
   
   nc <- sf::st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
   p <- ggplot(nc) + geom_sf() 
@@ -86,6 +89,7 @@ test_that("sf aspect ratio is correct", {
 
 test_that("works with a blank theme", {
   skip_if_not_installed("sf")
+  skip_if_not_installed("s2")
   
   nc <- sf::st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
   p <- ggplot(nc) + geom_sf() + 
