@@ -1,10 +1,9 @@
-context("size")
 
 test_that("sizemode is always respected", {
   
   # https://github.com/ropensci/plotly/issues/1133
   p <- plot_ly(mtcars, x = ~wt, y = ~wt, size = ~wt, color = ~as.factor(carb)) 
-  d <- plotly_build(p)$x$data
+  d <- expect_doppelganger_built(p, "sizemode")$data
   expect_length(d, length(unique(mtcars$carb)))
   
   for (i in seq_along(d)) {
@@ -16,5 +15,12 @@ test_that("sizemode is always respected", {
     s <- d[[i]]$marker$size
     if (length(s) == 1) expect_is(s, "AsIs")
   }
-  
+})
+
+
+test_that("size maps correctly to marker.size", {
+  p <- plot_ly(x = 1:10, y = 1:10, size = I(30))
+  d <- expect_doppelganger_built(p, "marker.size")$data
+  expect_length(d[[1]]$marker$size, 10)
+  expect_true(all(d[[1]]$marker$size == 30))
 })
